/* ************************************************************************** */
/** Descriptive File Name

  @ Author
    Richard Wall
 
  @ Date
    Created:    May 28, 2016
    Revision:   January 23, 2017 - added decimal point control
    Verified:   May 16, 2017

  @Company
    Digilent

  @File Name
    led7.c

  @ Date
    Created:    May 28, 2016
    Revision:   January 23, 2017 - added decimal point control
    Verified:   May 16, 2017

  @Summary
    Four digit - seven segment LED display library.

  @Description
    Displays decimal values of a four digit - seven segment display
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */
#include "swDelay.h"
#include "led7.h"
#include <plib.h>

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Four digit seven segment library                                  */
/* ************************************************************************** */
/* ************************************************************************** */

/* Provides control of individual digits or write a decimal value to the four
 * LED displays
 */

/** 
  @Function
    * void seg7_init(void);
  @Summary
    * Initialized 7 segment IO pins as outputs
  @Description
    * The assigned PIC32 IO pins are assigned as outputs and the digit LED 
    * anodes are set to zero.
  @Precondition
    * None
  @Parameters
    * None
  @Returns
    * None
  @Remarks
    * None
 */
void seg7_init(void)
{
    SET_CA();
    SET_CB();
    SET_CC();
    SET_CD();
    SET_CE();
    SET_CF();
    SET_CG();
    SET_DP();
    SET_AN0();
    SET_AN1();
    SET_AN2();
    SET_AN3();

    clr_dsp();
    
    DIG_AN0(1);     // Segment digits are active low 
    DIG_AN1(1);
    DIG_AN2(1);
    DIG_AN3(1);   

    test_7seg_leds();
}

/** 
  @Function
    * void clr_dsp(void);
  @Summary
    * Turns off all LED segments 
  @Description
    * All 7 LEDs are set to zero.
  @Precondition
    * LED IO pins have been initialized as outputs
  @Parameters
    * None    
  @Returns
    * None
  @Remarks
    * None
 */
void clr_dsp(void)
{
    SEG_CA(0);      
    SEG_CB(0);
    SEG_CC(0);
    SEG_CD(0);
    SEG_CE(0);
    SEG_CF(0);
    SEG_CG(0);
    SEG_DP(0);
}

/** 
  @Function
    * void set_digit(int dsp, int value);
  @Summary
    * Turns off all LED segments 
  @Description
    * All 7 LEDs are set to zero.
  @Precondition
    * LED IO pins have been initialized as outputs
  @Parameters
    * 1: Display digit
    * 2: Display value   
  @Returns
    * None
  @Remarks
    * Digital display anode controls are active low.
    * 
    * Pmod JA Pin 2 is used for timing instrumentation and should be removed 
    * after testing.
 */
void set_digit(int dsp, int value, int dp)
{
    dsp_digit(value, dp);   // Set LED cathodes according to 
    switch(dsp)
    {
        case 0:
            DIG_AN0(0);
            DIG_AN1(1);
            DIG_AN2(1);
            DIG_AN3(1);
            break;
        case 1:
            DIG_AN0(1);
            DIG_AN1(0);
            DIG_AN2(1);
            DIG_AN3(1);
            break;
        case 2:
            DIG_AN0(1);
            DIG_AN1(1);
            DIG_AN2(0);
            DIG_AN3(1);
            break;
        case 3:
            DIG_AN0(1);
            DIG_AN1(1);
            DIG_AN2(1);
            DIG_AN3(0);
            break;
        default:
            DIG_AN0(0);
            DIG_AN1(0);
            DIG_AN2(0);
            DIG_AN3(0);
    }
}

/** 
  @Function
* void dsp_digit(int value);

  @Summary
    *  Single digit LED display
  @Description
    * Turns on specific LEDs segments to display the value in decimal format.
  @Precondition
    * LED IO pins have been initialized as outputs
  @Parameters
    * int - value to be displayed    
  @Returns
    * None
  @Remarks
    * Each value is displayed using a macro that consists of multiple C 
    * statements.
 */
void dsp_digit(int value, int dp)
{
    switch(value)
    {      
        case 0:
            SEG_LED0
            break;
        case 1:
            SEG_LED1
            break;
        case 2:
            SEG_LED2
            break;
        case 3:
            SEG_LED3
            break;
        case 4:
            SEG_LED4
            break;
        case 5:
            SEG_LED5
            break;
        case 6:
            SEG_LED6
            break;
        case 7:
            SEG_LED7
            break;
        case 8:
            SEG_LED8
            break;
        case 9:
            SEG_LED9
            break;
        case -1:
            SEG_NEG     // generate negative sign
            break;            
        default:
            SEG_OFF    // Blank the display
    }
    SEG_DP(dp);
}

/** 
  @Function
* void led_number(int value);
  @Summary
    * Display 4 digit decimal number
  @Description
    * A value of -999 through 9999 is displayed on the 4 digit 7 segment 
    * display. Leading zero blanking is used.
  @Precondition
    * LED IO pins have been initialized as outputs 
  @Parameters
    * int - value to be displayed in the range -999 to +9999
    * None    
  @Returns
    * None
  @Remarks
    * Each value is displayed using a macro that consists of multiple C 
    * statements. This function must be call frequently to keep the numbers
    * visible to the viewer. 
 */
void led_number(int value)
{
int d1, d2, d3, d4, dz = 0;         // Segment values and display zero flag
    
    if(value<10000)                 // Test for less than maximum positive
    {
        if((value < 0) && ( value > -1000)) // Test maximum negative
        {
            value = -value;         // Negate value to make positive
            d4 = -1;                // Assign digit 4 to negative 1
        }
        else
        {
            d4 = value/1000;        // Isolate 1000's digit
            value = value % 1000;   // Save remainder
        }
        d3 = value/100;             // Isolate 100's digit
        value = value % 100;        // Save remainder
        d2 = value/10;              // Isolate 10's digit
        value = value % 10;         // Save remainder
        d1 = value;                 // Isolate units digit

        if(d4)                      // Test for non zero
        {
            set_digit(3, d4, 0);    // If non zero or minus sign - display digit
            if(d4>0)
                dz = 1;             // Set display zero flag
        }
        else
            set_digit(3, 0xFF, 0);     // If leading zero - blank digit
        msDelay(1);

        if(d3 || dz == 1)           // Display if nonzero or dz flag is set
        {
            set_digit(2, d3, 0);
            dz = 1;                 // Set display zero flag
        }
        else
            set_digit(2, 0xFF, 0);     // If leading zero - blank digit
        msDelay(1);

        if(d2 || dz == 1)           // Display if nonzero or dz flag is set
        {
            set_digit(1, d2, 0);       // Set display zero flag
        }
        else
            set_digit(1, 0xFF, 0);     // If leading zero - blank digit
        
        msDelay(1);
        set_digit(0, d1, 0);       // Display trailing zeros
        msDelay(1);

/* Blank the display */        
        DIGITS_OFF 
        SEG_OFF
    }
}

/** 
  @Function
* void test_7seg_leds(void);
  @Summary
    * A display test sequence for the 4 digit seven segment display
  @Description
    * Each of the 7 LED segments A through F are displayed on digit 0. The the 
    * number "8" is shifted from eft to right on the display 
  @Precondition
    * LED IO pins have been initialized as outputs 
  @Parameters
* int - value to be displayed
    * None    
  @Returns
    * None
  @Remarks
    * Each value is displayed using a macro that consists of multiple C 
    * statements.
 */
void test_7seg_leds(void)
{
    DIG_AN0(0);
    SEG_CA(1);
    msDelay(100);
    SEG_CB(1);
    msDelay(100);
    SEG_CC(1);
    msDelay(100);
    SEG_CD(1);
    msDelay(100);
    SEG_CE(1);
    msDelay(100);
    SEG_CF(1);
    msDelay(100);
    SEG_CG(1);
    msDelay(100);
    SEG_DP(1);
    msDelay(250);
    DIG_AN0(1);
    DIG_AN1(0);
    msDelay(250);
    DIG_AN1(1);
    DIG_AN2(0);
    msDelay(250);
    DIG_AN2(1);
    DIG_AN3(0);
    msDelay(200);
    clr_dsp();
    DIG_AN3(1);   
}
/* *****************************************************************************
 End of File
 */
