/* ************************************************************************** */
/** Descriptive File Name
  @ Author
    Richard Wall
 
  @ Date
    Created:    June 24, 2016
    Verified:   May 17, 2017

  @Company
    Digilent Inc.      

  @File Name
    MMA8652FC.h

  @Development Environment
    MPLAB X IDE x3.61 - http://www.microchip.com/mplab/mplab-x-ide 
	XC32 1.43 - http://www.microchip.com/mplab/compilers
	PLIB 3/7/20162 - http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library

  @Summary
    Interface with MMA8652FC accelerometer with I2C 

  @Description
   Initialization and sensor interface functions for the MMA8652FC accelerometer

 **************************************************************************** */

#ifndef __MMA8652FC_H__    /* Guard against multiple inclusion */
#define __MMA8652FC_H__

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    #define XYZ_ADDR        0X1D

    #define XYZ_STATUS      0x00
    #define XYZ_OUT_X_MSB   0x01
    #define XYZ_OUT_X_LSB   0x02
    #define XYZ_OUT_Y_MSB   0x03
    #define XYZ_OUT_Y_LSB   0x04
    #define XYZ_OUT_Z_MSB   0x05
    #define XYZ_OUT_Z_LSB   0x06

    #define XYZ_F_SETUP     0X0A
    #define XYZ_TRIG_CFG    0X0B
    #define XYZ_SYSMOD      0X0C
    #define XYZ_INT_SRC     0X0D
    #define XYZ_ID          0X0E
    #define XYZ_DATA_CFG    0X0F
    #define XYZ_HP_CO       0X10
    #define XYZ_PL_STATUS   0X11
    #define XYZ_PLCFG       0X12
    #define XYZ_PL_COUNT    0X13
    #define XYZ_BF_ZCOMP    0X14
    #define XYZ_P_L_THSREG  0X15
    #define XYZ_FF_MT_CFG   0X16
    #define XYZ_FF_MT_SRC   0X17
    #define XYZ_FF_MT_THS   0X18
    #define XYZ_MT_COUNT    0X19

    #define XYZ_CTRL_REG1   0x2A
    #define XYZ_CTRL_REG2   0x2B

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

/** Descriptive Data Type Name

  @Summary
    I2C Block Read and Block Write structure
    
  @Description
    Structure to facilitate passing one or more BYTES of data to or reading 
    from an I2C device. The following parameters are used:
        1. I2C channel (I2C1 through I2C4)
        2. 7 bit I2C device ID
        3. Devices register address to start read from
        4. pointer to memory for supplying or storing the data
        <p>
         The I2C channel must first be initialized before any MMA9652 
        accelerometer functions are called.
        <p>
          This structure is declared in  I2CLib.h
        <p>
   typedef struct 
    {
        I2C_MODULE i2c_channel; // I2C channel ? I2C1 or I2C2
        BYTE dev_id;            // I2C device ID
        BYTE reg_addr;          // Address of register to start
        BYTE block_size;        // Number of bytes to read or write
        BYTE *data;             // Byte pointer to data array
    } DATA_BLOCK
*/

#endif

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************
/**
  @Function
    void mma8562fc_init(void) 

  @Summary
    Initializes the I2C interface and MMA8652FC configuration registers

  @Description
    Initialization sequence for MMA8653FC accelerometer using a series of
    I2C messages.

  @Precondition
    I2C channel must be previously initialized

  @Parameters
    None
    
  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    All I2C operations must be successful for this function to complete 
    correctly.  I2C communications must have been previously initialized.

  @Example
    @code
        mma8562fc_init();

***************************************************************************** */
I2C_RESULT mma8562fc_init(void);

// *****************************************************************************
/** 
  @Function
    I2C_RESULT mma8562fc_xyz(int *status, int *x, int *y, int *z); 

  @Summary
    Reads the X, Y, and Z 12 bit parameters from the accelerometer

  @Description
    Reads the accelerometer data as an array of bytes and parses this array
    into the parameter values.

  @Parameters
    @param param1 - pointer to accelerometer status value
    @param param2 - pointer to accelerometer x axis value
    @param param3 - pointer to accelerometer y axis value
    @param param4 - pointer to accelerometer z axis value

  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    Reports success or I2C operation as an indication of valid data.

  @Example
    @code
        I2C_RESULT okay;
        int status, x,y,z;
        okay = mma8562fc_xyz(&status, &x, &y, &z);

***************************************************************************** */
I2C_RESULT mma8562fc_xyz(int *status, int *x, int *y, int *z);

// *****************************************************************************
/** 
  @Function
    I2C_RESULT mma8562fc_rd_reg(int reg, int *reg_value); 

  @Summary
    Reads a single byte register

  @Description
    Reads a single byte from the specified accelerometer register.

  @Parameters
    @param param1 - accelerometer register value
    @param param2 - pointer to accelerometer register value

  @Returns
    I2C results - I2C_SUCCESS / I2C_FAILURE 

  @Remarks
    Reports success or I2C operation as an indication of valid data.

  @Example
    @code
        I2C_RESULT okay;
        int reg = 1, reg_value;
        okay = mma8562fc_xyz(reg, &reg_value);

***************************************************************************** */
I2C_RESULT mma8562fc_rd_reg(int reg, int *reg_value);

/* *****************************************************************************
 End of File
 */
