/* ************************************************************************** */
/** Descriptive File Name:  Header file for SPI Flash Memory Driver

   @Company
    Digilient
 
  @Author
    Richard Wall

  @Date
    Created:    August 11, 2013
    Verified:   May 20, 2017

  @File Name
    spi.h

  @Summary
    Serial peripheral interface (SPI) communications.

 ************************************************************************** */
   
#ifndef __SPI_H__
    #define __SPI_H__

    #define PIC32MX_SPI1_SDO_SCK_MASK_VALUE (0x00000140)
    #define PIC32MX_SPI1_SDI_MASK_VALUE     (0x00000080)
    #define MAX_SPI_CLK_FREQ                1000000

    #define SPI1_SDI                        PORTFbits.RF7       // FLASH SO
    #define SDI1_TRIS                       TRISFbits.TRISF7    //
    #define SPI1_SDO                        LATFbits.LATF2      // FLASH SI
    #define SDO1_TRIS                       TRISFbits.TRISF2    //
    #define SPI1_SCK                        LATFbits.LATF6      // OK
    #define SCK1_TRIS                       TRISFbits.TRISF6    // OK
     
#endif

/** 
  @Function
    unsigned int init_SPI1(void); 

  @Summary
    Initializes SPI channel 1 for send and receive operation.

  @Description
    The optimal divisor is computed that results in the greatest SPI bit rate 
    to be generated that is closest to but less than the specified maximum
    bit rate. The SPI channel is enabled for Master MODE 0 operation. 

   @Precondition
    Maximum SPI bit rate must be defined
 
  @Parameters
    None
    
  @Returns
    Actual SPI bit rate.
  
  @Remarks
    None
 
  @Example
    @code
    {
        unsigned int spi_br = init_SPI1();
    }
 */
unsigned int init_SPI1(void);

/** 
  @Function
    void close_SPI1(void); 

  @Summary
    Disables the SPI1 channel.

  @Description
    No data written to the SPI1 buffer will be shifted out the SDO1 pin and 
    the SCK1 clock will not be generated.

   @Precondition
    None
 
  @Parameters
    None
    
  @Returns
    None
  
  @Remarks
    None
  @Example
    @code
    {
        close_SPI1();
    }
 */
void close_SPI1(void);

/** 
  @Function
    BYTE spiXfer(BYTE data_out); 

  @Summary
    Sends and receives one SPI byte.

  @Description

    This routine writes and reads a single byte/half word/word to the SPI bus.
    Initially, it waits for TX buffer empty, so that it doesn't overwrite the 
    previous untransmitted data.
    <p>    
    When a new data word has been shifted into shift register SPIxSR and
    the previous contents of receive register SPIxRXB have not been read
    then the Rx Overflow (SPIROV) bit will be set.
    The module will not transfer the received data from SPIxSR to the SPIxRXB.
    Further data reception is disabled until the SPIROV bit is cleared.
    The SPIROV bit is not cleared automatically by the module and must be
    cleared by the user software (using SpiChnGetRov()).

  @Precondition
    SPI channel must have been initialized.
    Chip must have been enable before calling this function.
    Chip Enable signal must be asserted high outside this function

  @Parameters
    @param param1 BYTE variable to be transmitted.
    
  @Returns
    None
  
  @Remarks
    Function blocks until buffer is empty. BYTE data type is the same as 
    unsigned char data type.

  @Example
    @code
    {
        BYTE char bVal = spiXfer(unsigned char bVal);
    }
 */
BYTE spiXfer(BYTE data_out);
