/* ************************************************************************** */
/** Descriptive File Name: motor.c is the preemptive motor speed control

  @Company
    Digilent Inc.  
 
  @Author
    Dr. Richard W Wall
 
  @Date
    Created: August 17, 2016
    Revised: March 2, 2017 - Implements both Lab 6a and 6b using SW0 control

  @File Name
    motor.c

  @Summary
    DC motor speed control using PWM.

  @Description
    These functions allow direct speed control of the DC motor.
 
  @Remarks
    Uses XC32 peripheral library. See 
 http://ww1.microchip.com/downloads/en/DeviceDoc/32bitPeripheralLibraryGuide.pdf
 http://www.microchip.com/SWLibraryWeb/product.aspx?product=PIC32%20Peripheral%20Library
 
 ****   This application uses a peripheral bus clock set to 10MHz.  ****
 
  *************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

// System Included files
#include "hardware.h"
#include <plib.h>
#include <stdio.h>
#include <inttypes.h>

// Application included files
#include "motor.h"
#include "lab6.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

float rps = 1.0;	// Motor Speed as determined in the Capture1 ISR for LCD 
int set_speed;      // Analog Control Input speed
int cn;             // Control voltage
int en;             // Control error
int adc2;           // Potentiometer ADC value

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Public Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/** 
  @Function
    void motor_init(void); 

  @Summary
    DC Motor Driver and tachometer initialization.

  @Description
    Configures the PWM outputs for the H Bridge motor driver using outputs for
    channels AIN only. Output compare channels 2 and 4 drive driver inputs AIN1
    and AIN2 respectively. Driver IC mode pine is set to zero.
     
    Timer 2 is used for the PWM time base and sets the PWM carrier frequency 
    for 10 KHz. Outputs AIN1 and AIN2 can be displayed on the Analog Discovery
    DIO 12 and 13.
     
    The Tachometer uses input capture channel 1 that uses Timer 3 for the 
    time reference. The Input capture pin is RG9 (Pmod JA:10) and has the 
    internal pull up resistor enabled. The SPI SCK pin is toggled each time 
    the input capture is triggered to provide a tachometer test point that can
    be displayed on the Analog Discovery  DIO 4.

  @Precondition
    None
    
  @Parameters
    @param param1 
        None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled
  @Example
    @code
    {
        motor_init(void); 
    }
 */
void motor_init(void) 
{

// configure to sample AN2 as the speed control
   SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN2); 
// configure ADC using parameter define above
   OpenADC10( ADC_PARAM1, ADC_PARAM2, ADC_PARAM3, ADC_PARAM4, ADC_PARAM5 ); 
   EnableADC10();      // Enable the ADC
    
    MCInit();           // Set motor control pins for output
            
// Turn power off to all phases
    MCMODE(0);          // Set MODE to zero (DC motor)
    AIN1(0);            // AIN1
    AIN2(0);            // AIN2
    BIN1(0);            // BIN1
    BIN2(0);            // BIN2
    
    RPB3R = PPS_RB3_OC4;            // Map RB3 to OC4 for PWM  - AIN1 - RB3   
    RPE8R = PPS_RE8_OC2;            // Map RE8 to OC2 for PWM  - AIN2 - RE8     
//  H-Bridge driver B is not used
//  RPE9R = PPS_RE9_OC5;            // Map RE9 to OC5 for PWM  - BIN1 - RE9     
//  RPB5R = PPS_RB5_OC3;            // Map RB5 to OC3 for PWM  - BIN2 - RB5     
    
    OpenTimer2((T2_ON | T2_SOURCE_INT | T2_PS_1_4), PWM_PERIOD-1);
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_1);

// H Bridge Driver A    
    OpenOC2((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
            0, 0);  // AIN2
    OpenOC4((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
            0, 0);  // AIN1
    
// H Bridge Driver B
//    OpenOC5((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
//            0, 0);  // BIN1
//    OpenOC3((OC_ON|OC_TIMER_MODE16|OC_TIMER2_SRC|OC_PWM_FAULT_PIN_DISABLE), 
//            0, 0);  // BIN2
    
    mIC1ClearIntFlag();
//    ConfigIntTimer3(T3_INT_ON | T3_INT_PRIOR_2 | T3_INT_SUB_PRIOR_0);
//    mT3IntEnable(1);                //  EnableIntT3
    
// Tachometer input pin configuration
    PORTSetPinsDigitalIn(IOPORT_G, TACHbit);    // JA10 - RG9
    PORTSetPinsDigitalOut(IOPORT_F, TACHout);   // Instrumentation only
    
    // Enable Input Capture Module 1
    // - Capture every rising edge
    // - Enable capture interrupts
    // - Use Timer 3 source
    // - Capture rising edge first
    ANSELGbits.ANSG9 = 0;           // Set pin as digital IO
    TRISGbits.TRISG9 = 1;           // Set pin as input
    
// Enable pull-up resistor for tachometer open collector output
    ConfigCNGPullups(CNG9_PULLUP_ENABLE);   // Enable  Pull Up
    IC1R = 0b00000001;                      // RG9 to Input Capture 1
    
// Configure input capture
    OpenTimer3( T3_ON | T3_PS_1_16, T3_TICK-1); 
    OpenCapture1( IC_EVERY_RISE_EDGE | IC_INT_1CAPTURE | IC_TIMER3_SRC |\
                  IC_FEDGE_RISE | IC_ON );  
    ConfigIntCapture1(IC_INT_ON | IC_INT_PRIOR_2 | IC_INT_SUB_PRIOR_0);

// Enable global nested interrupts    
    INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR); // Execute only once
    INTEnableInterrupts();  // Execute as needed

}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/*  DC Motor Control */

// *****************************************************************************

/** 
  @Function
    void motor(int control);
 
   @Summary
    Implements the motor control using the PWM.

  @Description
    This determines the motor operation mode based on the value of the  "mc"
    parameter. The COAST and BRAKE modes provide no motor output power. The
    The CW and CCW determine the direction of motor rotation and the value of 
    "ms" determines the percent PWM duty cycle to control the motor speed.

  @Precondition
    Processor pins for AIN1 and AIN2 must have been set for digital output. 
    See motor_init();
    
  @Parameters
    @param param1 
        Control -> -1 = CCW, 0 = OFF, 1 = CW
    @param param2
        ms - PWM output in % duty cycle
 
  @Returns
    None
 
  @Remarks
    None

  @Example
    @code
    { 
        motor(MOTOR_CW, 50); 
    }
 */
static void motor(enum MOTOR_CTRL mc, int ms)
{
    switch (mc)
    {
        case COAST:
            MOTOR_COAST();
            break;
        case CW:
            MOTOR_CW(ms);
            break;
        case CCW:
            MOTOR_CCW(ms);
            break;
        case BRAKE:
            MOTOR_BRAKE();
            break;
    }
}

/** 
  @Function
    void __ISR( _INPUT_CAPTURE_1_VECTOR, IPL2SOFT) Capture1(void);
 
   @Summary
    Input capture (IC) interrupt service routine(ISR).

  @Description
    The IC-ISR implements the motor tachometer by measuring the period of the
    digital input signal. This implementation uses a moving average (MA) low 
    pass filter to reduce the measurement sensitivity to mechanical and 
    electrical noise. The order of the MA filter is specified by the defined 
    constant IC1_SAMPLES.
    
  @Precondition
    Timer 3 is used as the time base. Conversion from timer counts to RPS depend
    on the Timer 3 prescale and the peripheral bus clock (PBCLK). 
    
  @Parameters
    None - ISRs cannot have any variables passed to them
 
  @Returns
    None - ISRs cannot return a value
 
  @Remarks
    Filter performance
        6.7us for IIR MA filter 
        11.5us for FIR MA-4 filter 
        18.0us for FIR MA-8 filter 

  @Example
    Execution of ISR code is initiated by the interrupting event.  
*/
void __ISR( _INPUT_CAPTURE_1_VECTOR, IPL2SOFT) Capture1(void)
{
static unsigned int con_buf[IC1_SAMPLES] = {0};	// Capture buffer 
static unsigned short int time1;                // Most recent captured time 
static unsigned short int time2 = 0;            // Previous time captured 
static short unsigned int time_diff = 0;        // Time between captures 
static int i1 = 0;                              // Averaging index 
static int iN = 1;
static float y1 = 0, y2 = 0;
static int x[IC1_SAMPLES + 1] = {0};
int i;
    
    PORTSetBits(IOPORT_F, TACHout); // Timing signal
    
    ReadCapture1(con_buf);          // Read captures into buffer 

    time1 = con_buf[0];             // Save time of event
    
    time_diff = (time1 - time2);    // Determine period from successive captures
    if(time_diff == 0)              // Test for possible divide by 0
    {
        time_diff = 1;              
    }
    else
    {
        x[i1] = time_diff;          // Save period in MA buffer

//  Moving average algorithm y[n] = (x[n-0 - x[n-N])/N  +  y[n-1]
        y1 = ((float)(x[i1] - x[iN])/IC1_SAMPLES) + y2; //Compute moving average

// Remember this value as y[n-1]
        y2 = y1;
        
        rps = TICKS2RPS / y1;      // Convert period to Hz

//  Generate circular buffer
        ++i1;   i1 = i1 % (IC1_SAMPLES+1);
        ++iN;   iN = iN % (IC1_SAMPLES+1);

        time2 = time1;		// Remember this time for next capture 
    }
    PORTClearBits(IOPORT_F, TACHout);       // Timing signal
    mIC1ClearIntFlag();                     // CLear interrupt flag
} // End of Capture1 ISR 

/** 
  @Function
    void __ISR(_TIMER_2_VECTOR, IPL3SOFT) Timer2Handler(void);
 
   @Summary
    Motor speed control.

  @Description
    The TImer 2 ISR implements both the open and closed loop control depending
    on the state on SW0 slide switch. When SW0 is off (0), the ISR implements
    open loop control. When SW0 is on (1), the ISR implements proportional plus
    integral (PI) closed loop control.
    
  @Precondition
    Timer 2 is used as the time base. Conversion from timer counts to RPS depend
    on the Timer 2 prescale and the peripheral bus clock (PBCLK). 
    
  @Parameters
    None - ISRs cannot have any variables passed to them
 
  @Returns
    None - ISRs cannot return a value
 
  @Remarks

  @Example
    Execution of ISR code is initiated by the interrupting event.  
*/
void __ISR(_TIMER_2_VECTOR, IPL3SOFT) Timer2Handler(void)
{
static float en1 = 0, en2 = 0, cn1 = 0, cn2 = 0;
static int start_delay = 500;   // 1/2 second control delay
int offset;
float rpsp;
static t2Clk = 1000;            // 1 second timer
int sw;

    if(--t2Clk == 0)    // Blink LED 0 at a 1 second rate for "alive" indication
    {
        invLED3();
        t2Clk = 1000;
    }
// Read ADC10 Channel 2
    offset = 8 * ((~ReadActiveBufferADC10() & 0x01));  
    adc2 = ReadADC10(offset);    
    
    if(SW0() == CLOSED_LOOP)
    {
// Scale input with bias
        set_speed = ((adc2 * 60 / ADCMAX) + 30);    // Compute percent speed
        if(!start_delay)
        {
// PI algorithm

// Compute present error - global rps is set by CCP ISR
            rpsp = (rps*100)/MAX_RPS;           // Compute percent rps
            en1 = set_speed - rpsp;             // Present error

// Compute present control output. Ka, Kb and Kgain are PI controller constants
// Values for cn2 and en2 are previously computed
            cn1 = cn2 + ((en1*Ka) - (en2*Kb))/Kgain;

 // Limit control output to prevent integrator windup
            if(cn1 > 100)
                cn1 = 100;
            if(cn1 < 0)
                cn1 = 0;

// Save computed control output and error for next itteration
            cn2 = cn1;
            en2 = en1;

// Global variables used for display output         
            en = (int) en1;     // Used for display        
            cn = (int) cn1;     // Control parameter and used for display

// Limit control output to between 0 and 100%
            if((cn1 <= 100) && (cn1 >= 0))
            {
                cn = (int) ((cn1  * PWM_PERIOD)/100);        
            }
            else
            {
                if(cn1 >= 100)
                {
                    cn = PWM_PERIOD;
                }
                if(cn1 < 0)
                {
                    cn = 0;
                }
            }
        }
        else    // Initialize by letting motor run open loop
        {
            cn = (int) (set_speed * PWM_PERIOD)/100 ;
            --start_delay;
        }
    }
    else                                // Set motor speed for OPEN_LOOP
    {
        set_speed = (adc2 * 100) / ADCMAX;    // Compute percent speed
        cn = (int) (set_speed * PWM_PERIOD)/100 ;
    }

    sw = Switch2Binary(); 
    sw &= 0xE0; // Decode SW6 through SW7 for motor run mode
    switch(sw)
    {
        case 0b00100000:        // SW5
            motor(COAST, cn);
            break;
        case 0b01000000:        // SW6
            motor(BRAKE, cn);
            break;
        case 0b10000000:        // SW7
            motor(CW, cn);
            break;
        case 0b00000000:        // No switches
            motor(CCW, cn);        
            break;               
    }

    mT2ClearIntFlag();          // Clear interrupt flag
}

/** 
  @Function
    void __ISR(_TIMER_3_VECTOR, IPL2SOFT) Timer3Handler(void);
 
   @Summary
    Unused Timer 3 rollover interrupt.

  @Description
    
  @Precondition
    Timer 3 is used as the time base. . 
    
  @Parameters
    None - ISRs cannot have any variables passed to them
 
  @Returns
    None - ISRs cannot return a value
 
  @Remarks
    Available for future applications

  @Example
    Execution of ISR code is initiated by the interrupting event.  
*/
void __ISR(_TIMER_3_VECTOR, IPL2SOFT) Timer3Handler(void)
{
/* User supplied code */
    mT3ClearIntFlag();
}

/* *****************************************************************************
 End of File motor.c
 */
