/* ************************************************************************** */
/** Descriptive File Name: motor.h is the header file for motor.c. This declares
    file constants and public functions that provide global access to selected
    motor.c functions. The motor control is a preemptive system therefore access
    to the real-time function is provided by the Timer 2 and Timer 3. The only
    required public access is the motor initialization function. 

  @Company
    Digilent   
 
  @Author
    Richard W Wall
 
  @Date
    Created:    August 17, 2016
    Verified:   May 20, 2017
 
  @File Name
    motor.h

  @Summary
    PWM DC motor control.

  @Description
     This file sets the motor speed using PWM
 */
/* ************************************************************************** */

#ifndef __DC_MOTOR_H__    /* Guard against multiple inclusion */
#define __DC_MOTOR_H__

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

#include "hardware.h"
#include <plib.h>
/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/** Descriptive Constant Name

  @Summary
    DC motor control parameters
    
  @Description
    
  @Remarks
    The motor driver IC allows the paralleling of driver outputs but to do
    so requires the inputs to also be in parallel.
*/
    #define A1INbit         BIT_3       // RB3
    #define A2INbit         BIT_8       // RE8
//  #define B1INbit         BIT_9       // RE9
//  #define B2INbit         BIT_5       // RB5
//  #define ENAbit          A2INbit     // RE8
//  #define ENBbit          B2INbit     // RB5
    #define MOTOR_MODEbit   BIT_1       // RF1

//  #define PHASEA(a);  {if(a) {LATBSET = A1INbit; else LATBCLR = A1INbit;}
//  #define PHASEB(a);  {if(a) {LATESET = B1INbit; else LATECLR = B1INbit;}
//  #define ENA(a);     {if(a) {LATESET = ENAbit; else LATECLR = ENAbit;}
//  #define ENB(a);     {if(a) {LATBSET = ENBbit; else LATBCLR = ENBbit;}

    #define PPS_RB3_OC4 0b00001011  // Map RB3 to OC4 for PWM     
    #define PPS_RE8_OC2 0b00001011  // Map RE8 to OC2 for PWM     
//  #define PPS_RE9_OC5 0b00001011  // Map RE9 to OC5 for PWM     
//  #define PPS_RB5_OC3 0b00001011  // Map RB5 to OC3 for PWM     

    #define PWM_PERIOD      (GetPeripheralClock()/1000)   // One ms period
    #define MOTOR_COAST();  {SetDCOC2PWM(0);SetDCOC4PWM(0);} 
    #define MOTOR_CW(a);    {SetDCOC4PWM(a);SetDCOC2PWM(0);} 
    #define MOTOR_CCW(a);   {SetDCOC4PWM(0);SetDCOC2PWM(a);}  
    #define MOTOR_BRAKE();  {SetDCOC2PWM(PWM_PERIOD);SetDCOC4PWM(PWM_PERIOD);}  

//  #define MOTOR_COAST();  {SetDCOC2PWM(0);SetDCOC3PWM(0);SetDCOC4PWM(0);SetDCOC5PWM(0);} 
//  #define MOTOR_CW(a);    {SetDCOC4PWM(a);SetDCOC5PWM(a);SetDCOC2PWM(0);SetDCOC3PWM(0);} 
//  #define MOTOR_CCW(a);   {SetDCOC4PWM(0);SetDCOC5PWM(0);SetDCOC2PWM(a);SetDCOC3PWM(a);}  
//  #define MOTOR_BRAKE();  {SetDCOC2PWM(PWM_PD);SetDCOC3PWM(PWM_PD);SetDCOC4PWM(PWM_PD);SetDCOC5PWM(PWM_PD);}  

    #define TACHbit         BIT_9   // RG9 - JA10
    #define TACHout         BIT_6   // RF6 - SPI_SCK - test only
    #define T3_TICK         0       // Maximum period
    #define PD2RPS          (GetPeripheralClock()/16)

// (includes prescale) 
    #define IC1_SAMPLES     8       // Moving average filter order
    #define TIMER_3_RATE    1
    #define TICKS2RPS       625000  // Conversion of timer ticks to RPS

    #define CLOSED_LOOP     (int) 1
    #define OPEN_LOOP       (int) 0

    #define Ka              64      // PI control gains
    #define Kb              60      // 60  
    #define Kgain           (1 << 10) // Common divisor (Inverse gain))

    #define MAX_RPS         300

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

    enum  MOTOR_CTRL {COAST=0, CW, CCW, BRAKE} ;

#endif /* __DC_MOTOR_H__ */

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/* ****************************************************************************
/** 
  @Function
    void motor_init(void); 

  @Summary
    Initializes the PIC32 motor control pins for output

  @Description
     Configures the PWM outputs for the H Bridge motor driver using outputs for
     channels AIN only. Output compare channels 2 and 4 drive driver inputs AIN1
     and AIN2 respectively. Driver IC mode pine is set to zero.
     
     Timer 2 is used for the PWM time base and sets the PWM carrier frequency 
     for 10 KHz. Outputs AIN1 and AIN2 can be displayed on the Analog Discovery
     DIO 12 and 13.
     
     The Tachometer uses input capture channel 1 that uses Timer 3 for the 
     time reference. The Input capture pin is RG9 (Pmod JA:10) and has the 
     internal pull up resistor enabled. The SPI SCK pin is toggled each time 
     the input capture is triggered to provide a tachometer test point that can
     be displayed on the Analog Discovery  DIO 4.

  @Precondition
    None
    
  @Parameters
    None
  @Returns
    None
  @Remarks
    All analog capable IO pins must have the analog functionality disabled.
    Motor driver inputs BIN1 and BIN2 are not used.

  @Example
    @code
      motor_init(); 
 */
void motor_init(void);


/* *****************************************************************************
 End of File
 */
